<?php
/**
 * @brief		File Storage Extension: File
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Chatbox+
 * @since		24 Jan 2020
 */

namespace IPS\chatbox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * File Storage Extension: File
 */
class _File
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'chatbox_files' )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\UnderflowException					When file record doesn't exist. Indicating there are no more files to move
	 * @return	void|int							An offset integer to use on the next cycle, or nothing
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$record = \IPS\Db::i()->select( '*', 'chatbox_files', array(), 'file_id', array( $offset, 1 ) )->first();

		try
		{
			$file = \IPS\File::get( $oldConfiguration ?: 'chatbox_File', $record['file_location'] )->move( $storageConfiguration );

			if ( (string) $file != $record['file_location'] )
			{
				\IPS\Db::i()->update( 'chatbox_files', array( 'file_location' => (string) $file ), array( 'file_id=?', $record['file_id'] ) );
			}
			
			$file = \IPS\File::get( $oldConfiguration ?: 'chatbox_File', $record['file_thumb'] )->move( $storageConfiguration );
			
			if ( (string) $file != $record['file_thumb'] )
			{
				\IPS\Db::i()->update( 'chatbox_files', array( 'file_thumb' => (string) $file ), array( 'file_id=?', $record['file_id'] ) );
			}
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$record = \IPS\Db::i()->select( '*', 'chatbox_files', array(), 'file_id', array( $offset, 1 ) )->first();
		
		if ( $new = \IPS\File::repairUrl( $record['file_location'] ) )
		{
			\IPS\Db::i()->update( 'chatbox_files', array( 'file_location' => $new ), array( 'file_id=?', $record['file_id'] ) );
		}
		
		if ( $record['record_thumb'] and $new = \IPS\File::repairUrl( $record['record_thumb'] ) )
		{
			\IPS\Db::i()->update( 'chatbox_files', array( 'file_thumb' => $new ), array( 'file_id=?', $record['file_id'] ) );
		}		
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	string	$file		The file path to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			$record	= \IPS\Db::i()->select( '*', 'chatbox_files' )->first();

			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'chatbox_files' ) as $file )
		{
			try
			{
				\IPS\File::get( 'chatbox_File', $file['file_location'] )->delete();
			}
			catch( \Exception $e ){}
			
			if( $file['file_thumb'] )
			{
				try
				{
					\IPS\File::get( 'chatbox_File', $screenshot['file_thumb'] )->delete();
				}
				catch( \Exception $e ){}
			}
			
		}
	}
}