<?php
/**
 * @brief		File Storage Extension: Nofifile
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Forums
 * @since		06 Feb 2014
 */

namespace IPS\chatbox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * File Storage Extension: Nofifile
 */
class _Nofifile
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		$count = 2; // it's the number of steps
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		switch ( $offset )
		{
			case 0:
				foreach ( \IPS\chatbox\Rooms\Room::roots() as $room )
				{
					if ( $room->notifile )
					{
						try
						{
							$room->notifile = (string) \IPS\File::get( $oldConfiguration ?: 'chatbox_Nofifile', $room->notifile )->move( $storageConfiguration );
							$room->save();
						}
						catch( \Exception $e ){}
					}
				}
				
				return TRUE;

			case 1:
				$updated = array();
				
				if( \IPS\Settings::i()->chatbox_con_notifile )
				{
					try
					{
						$updated['chatbox_con_notifile'] = \IPS\File::get( $oldConfiguration ?: 'chatbox_Nofifile', \IPS\Settings::i()->chatbox_con_notifile )->move( $storageConfiguration );
					}
					catch( \Exception $e ) {}
				}
		
				if( \IPS\Settings::i()->room_clubs_notifile )
				{
					try
					{					
						$updated['room_clubs_notifile'] = \IPS\File::get( $oldConfiguration ?: 'chatbox_Nofifile', \IPS\Settings::i()->room_clubs_notifile )->move( $storageConfiguration );
					}
					catch( \Exception $e ) {}
				}	
					
				if( $updated && \count( $updated ) )
				{
					\IPS\Settings::i()->changeValues( $updated );
				}
				
				return TRUE;

			default:
				throw new \UnderflowException;
		}
	}
	
	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		switch ( $offset )
		{
			case 0:
				foreach ( \IPS\chatbox\Rooms\Room::roots() as $room )
				{
					if ( $room->notifile )
					{
						if ( $new = \IPS\File::repairUrl( $room->notifile ) )
						{
							try
							{
								$room->notifile = $new;
								$room->save();
							}
							catch( \Exception $e ){}
						}
					}
				}
				
				return TRUE;

			case 1:
				$updated = array();
				
				if( \IPS\Settings::i()->chatbox_con_notifile )
				{
					try
					{
						$updated['chatbox_con_notifile'] = \IPS\File::repairUrl( \IPS\Settings::i()->chatbox_con_notifile );
					}
					catch( \Exception $e ) {}
				}
				
				if( \IPS\Settings::i()->room_clubs_notifile )
				{
					try
					{					
						$updated['room_clubs_notifile'] = \IPS\File::repairUrl( \IPS\Settings::i()->room_clubs_notifile );
					}
					catch( \Exception $e ) {}						
				}	
				
				if( $updated && \count( $updated ) )
				{
					\IPS\Settings::i()->changeValues( $updated );
				}
				
				return TRUE;

			default:
				throw new \UnderflowException;
		}
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	string	$file		The file path to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		if( \IPS\Settings::i()->chatbox_con_notifile AND $file == \IPS\Settings::i()->chatbox_con_notifile )
		{
			return TRUE;
		}
		
		if( \IPS\Settings::i()->room_clubs_notifile AND $file == \IPS\Settings::i()->room_clubs_notifile )
		{
			return TRUE;
		}
		
		foreach ( \IPS\Db::i()->select( 'room_id', 'chatbox_rooms', array( 'room_notifile=?', $file ) ) as $room )
		{
			if ( $room['room_notifile'] == (string) $file )
			{
				return TRUE;
			}
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		if( \IPS\Settings::i()->chatbox_con_notifile )
		{
			try
			{
				\IPS\File::get( 'chatbox_Nofifile', \IPS\Settings::i()->chatbox_con_notifile )->delete();
			}
			catch( \OutOfRangeException $e ){}
		}
		
		if( \IPS\Settings::i()->room_clubs_notifile )
		{
			try
			{
				\IPS\File::get( 'chatbox_Nofifile', \IPS\Settings::i()->room_clubs_notifile )->delete();
			}
			catch( \OutOfRangeException $e ){}
		}
		
		foreach( \IPS\Db::i()->select( '*', 'chatbox_rooms', "room_notifile IS NOT NULL" ) as $room )
		{
			try
			{
				\IPS\File::get( 'chatbox_Nofifile', $room['room_notifile'] )->delete();
			}
			catch( \OutOfRangeException $e ){}
		}
	}
}